
/****************************************************************************
 * NCSA HDF                                                                 *
 * Scientific Data Technologies                                             *
 * National Center for Supercomputing Applications                          *
 * University of Illinois at Urbana-Champaign                               *
 * 605 E. Springfield, Champaign IL 61820                                   *
 *                                                                          *
 * For conditions of distribution and use, see the accompanying             *
 * hdf/COPYING f.                                                           *
 *                                                                          *
 ****************************************************************************/


#include "H5HL.h"
#include <stdlib.h>

/*-------------------------------------------------------------------------
 * Table API test
 *
 * Functions tested:
 *
 * H5TBmake_table 
 * H5TBdelete_record
 * H5TBread_table
 *
 *-------------------------------------------------------------------------
 */

#define NFIELDS  5
#define NRECORDS 8

#if 0
#define  HAVE_FILL
#endif

int main( void )
{
 typedef struct Particle 
 {
  char   name[16];
  int    lati;
  int    longi;
  float  pressure;
  double temperature; 
 } Particle;

 
 /* Calculate the size and the offsets of our struct members in memory */
 size_t dst_size =  sizeof( Particle );
 size_t dst_offset[NFIELDS] = { HOFFSET( Particle, name ),
                                HOFFSET( Particle, lati ),
                                HOFFSET( Particle, longi ),
                                HOFFSET( Particle, pressure ),
                                HOFFSET( Particle, temperature )};

  /* Define an array of Particles */
 Particle  p_data[NRECORDS] = { {"zero",0,0, 0.0f, 0.0},
 {"one",10,10, 1.0f, 10.0},
 {"two",  20,20, 2.0f, 20.0},
 {"three",30,30, 3.0f, 30.0},
 {"four", 40,40, 4.0f, 40.0},
 {"five", 50,50, 5.0f, 50.0},
 {"six",  60,60, 6.0f, 60.0},
 {"seven",70,70, 7.0f, 70.0}
  };

 size_t dst_sizes[NFIELDS] = { sizeof( p_data[0].name),
                               sizeof( p_data[0].lati),
                               sizeof( p_data[0].longi),
                               sizeof( p_data[0].pressure),
                               sizeof( p_data[0].temperature)};

  /* Define field information */
  const char *field_names[NFIELDS]  = 
  { "Name","Latitude", "Longitude", "Pressure", "Temperature" };
  hid_t      field_type[NFIELDS];
  hid_t      string_type;
  hid_t      file_id;
  hsize_t    chunk_size = 10;
  int        compress  = 0;
#if defined  (HAVE_FILL)
 Particle   fill_data[1] = 
 { "no data",-1,-1, -99.0f, -99.0 };
#else
 int        *fill_data = NULL;
#endif
  hsize_t    start;      /* Record to start reading */
  hsize_t    nrecords;   /* Number of records to insert/delete */
  hsize_t    nfields_out;
  hsize_t    nrecords_out;
  int        i;
  Particle   *dst_buf;

  /* Initialize the field field_type */
  string_type = H5Tcopy( H5T_C_S1 );
  H5Tset_size( string_type, 16 );
  field_type[0] = string_type;
  field_type[1] = H5T_NATIVE_INT;
  field_type[2] = H5T_NATIVE_INT;
  field_type[3] = H5T_NATIVE_FLOAT;
  field_type[4] = H5T_NATIVE_DOUBLE;

  TESTING("deleting records");
  
  /* Create a new file using default properties. */
  file_id = H5Fcreate( "delete_record.h5", H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT );

/*-------------------------------------------------------------------------
 * make 2 tables, and delete records from one of them
 *-------------------------------------------------------------------------
 */

 if ( H5TBmake_table( "Table Title", file_id, "Table1", (hsize_t)NFIELDS, (hsize_t)NRECORDS, 
                       dst_size, field_names, dst_offset, field_type, 
                       chunk_size, fill_data, compress, p_data  ) < 0 )
   goto out;

 if ( H5TBmake_table( "Table Title", file_id, "Table2",(hsize_t) NFIELDS, (hsize_t)NRECORDS,
                       dst_size, field_names, dst_offset, field_type, 
                       chunk_size, fill_data, compress, p_data  ) < 0 )
   goto out;
 
 /* Delete records  */
 start    = 3;      
 nrecords = 3; 
 if ( H5TBdelete_record( file_id, "Table2", start, nrecords ) < 0 )
  goto out;


 /* Get the number of records and fields  */
 if ( H5TBget_table_info ( file_id, "Table2", &nfields_out, &nrecords_out ) < 0 )
  goto out;

 if (nrecords_out!=NRECORDS-3)
  goto out;

 dst_buf=(Particle*)calloc((size_t)5, sizeof(Particle) );

 if ( H5TBread_table( file_id, "Table2", dst_size, dst_offset, dst_sizes, dst_buf ) < 0 )
  goto out;

 /* Compare the extracted table with the original array */
 for( i = 0; i < NRECORDS-3; i++ ) 
 {

  if ( i < 3 )
  {
    if ( ( strcmp( dst_buf[i].name,  p_data[i].name ) != 0 ) ||
         dst_buf[i].lati          != p_data[i].lati ||
         dst_buf[i].longi         != p_data[i].longi ||
         dst_buf[i].pressure      != p_data[i].pressure ||
         dst_buf[i].temperature   != p_data[i].temperature ) {
   goto out;
  }

  }

  else
  {
  if ( ( strcmp( dst_buf[i].name,  p_data[i+3].name ) != 0 ) ||
         dst_buf[i].lati        != p_data[i+3].lati ||
         dst_buf[i].longi       != p_data[i+3].longi ||
         dst_buf[i].pressure    != p_data[i+3].pressure ||
         dst_buf[i].temperature != p_data[i+3].temperature ) {
   goto out;
  }
  }
  
 }


/*-------------------------------------------------------------------------
 * end
 *-------------------------------------------------------------------------
 */
 
 /* Close the file. */
 H5Fclose( file_id );
 free( dst_buf );

 PASSED();

 return 0;

out:
 /* Close the file. */
 H5Fclose( file_id );
 H5_FAILED();
 return 1;
}

