
/****************************************************************************
 * NCSA HDF                                                                 *
 * Scientific Data Technologies                                             *
 * National Center for Supercomputing Applications                          *
 * University of Illinois at Urbana-Champaign                               *
 * 605 E. Springfield, Champaign IL 61820                                   *
 *                                                                          *
 * For conditions of distribution and use, see the accompanying             *
 * hdf/COPYING f.                                                           *
 *                                                                          *
 ****************************************************************************/


#include "H5HL.h"

#include <stdlib.h>


/*-------------------------------------------------------------------------
 * Table API test
 *
 * Functions tested:
 *
 * H5TBmake_table 
 * H5TBread_table
 * H5TBwrite_records
 * H5TBread_records
 *
 *-------------------------------------------------------------------------
 */


#define NFIELDS  5
#define NRECORDS 8

int main( void )
{
 
 typedef struct Particle 
 {
  char   name[16];
  int    lati;
  int    longi;
  float  pressure;
  double temperature; 
 } Particle;

	
 Particle  dst_buf[NRECORDS];
 
 /* Calculate the size and the offsets of our struct members in memory */
 size_t dst_size =  sizeof( Particle );
 size_t dst_offset[NFIELDS] = { HOFFSET( Particle, name ),
                                HOFFSET( Particle, lati ),
                                HOFFSET( Particle, longi ),
                                HOFFSET( Particle, pressure ),
                                HOFFSET( Particle, temperature )};

 size_t dst_sizes[NFIELDS] = { sizeof( dst_buf[0].name),
                               sizeof( dst_buf[0].lati),
                               sizeof( dst_buf[0].longi),
                               sizeof( dst_buf[0].pressure),
                               sizeof( dst_buf[0].temperature)};
   
 /* Initially no data */
 Particle  *p_data = NULL;

 /* Define field information */
 const char *field_names[NFIELDS]  = 
	{ "Name","Latitude", "Longitude", "Pressure", "Temperature" };
 hid_t      field_type[NFIELDS];
 hid_t      string_type;
 hid_t      file_id;
 hsize_t    chunk_size = 10;
 int        *fill_data = NULL;
 int        compress  = 0;
 int        i;
 hsize_t    start;      /* Record to start reading/writing */
 hsize_t    nrecords;   /* Number of records to read/write */

 /* Define 2 new particles to overwrite */ 
 Particle  particle_in[2] = 
	{{ "zero",0,0, 0.0f, 0.0},
	{"one",10,10, 1.0f, 10.0} };
 Particle  particle_out[2];

 /* Initialize the field field_type */
 string_type = H5Tcopy( H5T_C_S1 );
 H5Tset_size( string_type, 16 );
 field_type[0] = string_type;
 field_type[1] = H5T_NATIVE_INT;
 field_type[2] = H5T_NATIVE_INT;
 field_type[3] = H5T_NATIVE_FLOAT;
 field_type[4] = H5T_NATIVE_DOUBLE;


 TESTING("writing records");
  
 /* Create a new file using default properties. */
 file_id = H5Fcreate( "write_record_fill.h5", H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT );

 /* Make the table */
 if ( H5TBmake_table( "Table Title", file_id, "Table1",(hsize_t) NFIELDS, (hsize_t)NRECORDS, 
		                     dst_size, field_names, dst_offset, field_type, 
                       chunk_size, fill_data, compress, p_data  ) < 0 )
   goto out;
 

 /* Overwrite 2 records starting at record 0 */
 start    = 0;      
 nrecords = 2; 
 if ( H5TBwrite_records( file_id, "Table1", start, nrecords, dst_size, dst_offset, 
		particle_in ) < 0 )
  goto out;

 /* Read back the table */
 if ( H5TBread_table( file_id, "Table1", dst_size, dst_offset, dst_sizes, dst_buf ) < 0 )
  goto out;

 /* Compare the extracted table with the initial values */
 for( i = 0; i <= 1; i++ ) 
 {
  if ( ( strcmp( dst_buf[i].name,  particle_in[i].name ) != 0 ) ||
         dst_buf[i].lati        != particle_in[i].lati ||
         dst_buf[i].longi       != particle_in[i].longi ||
         dst_buf[i].pressure    != particle_in[i].pressure ||
         dst_buf[i].temperature != particle_in[i].temperature ) {
   goto out;
  }
 }
 /* Read just the written records */
 start    = 0;      
 nrecords = 2; 
 if ( H5TBread_records( file_id, "Table1", start, nrecords, dst_size, dst_offset, 
		particle_out ) < 0 )
  goto out;

 /* Compare with original array */
 for( i = 0; i < 2; i++) 
 {
  if ( ( strcmp( particle_out[i].name, particle_in[i].name ) != 0 ) ||
         particle_out[i].lati        != particle_in[i].lati ||
         particle_out[i].longi       != particle_in[i].longi ||
         particle_out[i].pressure    != particle_in[i].pressure ||
         particle_out[i].temperature != particle_in[i].temperature ) {
          goto out;
  }
 }

    
/*-------------------------------------------------------------------------
 * end
 *-------------------------------------------------------------------------
 */
 
 /* Close the file. */
 H5Fclose( file_id );

 PASSED();

 return 0;

out:
 /* Close the file. */
 H5Fclose( file_id );
 H5_FAILED();
 return 1;

}

